// **** LED Ring Watch with ATtiny85 ****

// **** Required libraries ****
#include <Adafruit_NeoPixel.h>
#include <ThreeWire.h>  
#include <RtcDS1302.h>

// **** Configuration ****
#define PIN_LED              3
#define PIN_RST              2
#define PIN_DAT              1
#define PIN_CLK              0

#define LED_COUNT           16
#define LED_BRIGHTNESS     127 // Overall brightness of the LEDs.
#define LOOP_MAX            84 // Number of cycles for the "glow" effect.
#define PIXEL_SHIFT         13 // Number of pixels to be shifted to make the digits display match the LEDs.

Adafruit_NeoPixel pixels = Adafruit_NeoPixel(LED_COUNT, PIN_LED, NEO_GRB + NEO_KHZ800);
ThreeWire myWire(PIN_DAT, PIN_CLK, PIN_RST);
RtcDS1302<ThreeWire> rtc(myWire);
long count = 0; 

// **** Initialization **** 
void setup(){
  int k;
  uint32_t totalSeconds; // Time in seconds since the 1st of January 2000.
  pixels.begin();
  rtc.Begin();
  // The next line will "capture" the date and time when the sketch was compiled.
  RtcDateTime compileTime = RtcDateTime(__DATE__, __TIME__);
  // If the time of the real-time clock is "smaller" than this, it is likely that its battery is empty and it lost track of the actual time.
  // Hence, the next lines will make the RTC chip running, and if it reports a "smaller" time, the code will update the RTC with the compile time.
  if(rtc.GetIsWriteProtected()){
    rtc.SetIsWriteProtected(false);
  }
  if(!rtc.IsDateTimeValid()){
    rtc.SetDateTime(compileTime);
  }
  if(!rtc.GetIsRunning()){
    rtc.SetIsRunning(true);
  }
  RtcDateTime now = rtc.GetDateTime();
  if(now < compileTime){
    compileTime += 120; // compileTime is the time written to the binary code during the compilation of the code.
                        // The ATtiny must be transfered from the programmer to the clock circuit, and the circuit must be powerd on before the
                        // date and time can be transfered to the real-time-chip.
                        // Hence, we add two minutes for this transfering/handling process. 
    rtc.SetDateTime(compileTime);
    // Set all LEDs to full blue for a second to show that the RTC has received a new time value.
    for(k = 0; k < 16; k++){
      pixels.setPixelColor(k, 0x0000FF); 
    }
    pixels.show(); 
    delay(1000;
    for(k = 0; k < 16; k++){
      pixels.setPixelColor(k, 0x000000); 
    }
    pixels.show(); 
  }
}

// **** Loop **** 
void loop(){
  unsigned long color;
  unsigned long coeff;
  long seconds, minutes, hours;
  int k;

  RtcDateTime now = rtc.GetDateTime();
  if(now.IsValid()){
    hours = now.Hour();
    minutes = now.Minute();
    seconds = now.Second();
  }
  else{
    // Failed to read the time --> Set all LEDs to RED to show the error.
    for(k = 0; k < 16; k++){
      pixels.setPixelColor(k, 0x0000FF); 
    }
    pixels.show(); 
    return;
  }
  for(k = 0; k < 6; k++){
    coeff = abs(((count + LOOP_MAX/6 * k) % LOOP_MAX) - LOOP_MAX/2);
    color = (coeff << 16) + (40 << 8);
    if(seconds & (0x01 << k)){
      pixels.setPixelColor((k + PIXEL_SHIFT) % 16, color);    
    }
    else{
      pixels.setPixelColor((k + PIXEL_SHIFT) % 16, 0x000000);    
    }
  }
  for(k = 6; k < 12; k++){
    coeff = abs(((count + LOOP_MAX/6 * k) % LOOP_MAX) - LOOP_MAX/2);
    color = ((coeff*2 + 40) << 16) + (coeff/3);
    if(minutes & (0x01 << (k-6))){
      pixels.setPixelColor((k + PIXEL_SHIFT) % 16, color);    
    }
    else{
      pixels.setPixelColor((k + PIXEL_SHIFT) % 16, 0x000000);    
    }
  }
  hours %= 12;
  if(hours == 0){
    hours = 12; // Midnight --> 0:00 = 12:00 a.m.; Noon --> 12:00 = 12:00 p.m.
  }
  for(k = 12; k < 16; k++){
    coeff = abs(((count + LOOP_MAX/4 * k) % LOOP_MAX) - LOOP_MAX/2);
    color = (((unsigned long)(coeff*1.5)) << 8) + (coeff + 40);
    if(hours & (0x01 << (k-12))){
      pixels.setPixelColor((k + PIXEL_SHIFT) % 16, color);    
    }
    else{
      pixels.setPixelColor((k + PIXEL_SHIFT) % 16, 0x000000);    
    }
  }
  pixels.show(); 
  delay(20);
  count = (count + 1)%LOOP_MAX;
}
